package com.example.test_plugin_fftt_test;

import android.content.Context;
import android.location.Location;
import android.util.Log;

import androidx.annotation.NonNull;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import org.json.JSONObject;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.teliver.sdk.core.InitStatusListener;
import io.teliver.sdk.core.TLog;
import io.teliver.sdk.core.Teliver;
import io.teliver.sdk.core.TripListener;
import io.teliver.sdk.models.MarkerOption;
import io.teliver.sdk.models.PushData;
import io.teliver.sdk.models.TrackingBuilder;
import io.teliver.sdk.models.Trip;
import io.teliver.sdk.models.User;
import io.teliver.sdk.util.TUtils;

/**
 * TestPluginFfttTestPlugin
 */
public class TestPluginFfttTestPlugin implements FlutterPlugin, MethodCallHandler
        , ActivityAware, InitStatusListener, TripListener {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private MethodChannel channel;

    private Context context;

    private Result result;

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "test_plugin_fftt_test");
        channel.setMethodCallHandler(this);
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        try {
            switch (call.method) {
                case "init":
                    this.result = result;
                    Teliver.init(context, call.argument("public_key").toString());
                    Teliver.setInitStatusListener(this);
                    break;
                case "identifyUser":
                    identifyUser(call.argument("user_data").toString());
                    break;
                case "unregisterUser":
                    Teliver.unregisterUser();
                    break;
                case "startTrip":
                    this.result = result;
                    Teliver.setTripListener(this);
                    Teliver.startTrip(call.argument("trip_id").toString());
                    break;
                case "stopTrip":
                    this.result = result;
                    Teliver.setTripListener(this);
                    Teliver.stopTrip(call.argument("trip_id").toString());
                    break;
                case "startTracking":
                    startTracking(call.argument("trip_id").toString());
                    break;
                case "startMultiTracking":
                    startMultiTracking(call.argument("trip_ids").toString());
                    break;
                case "stopTracking":
                    stopTracking(Collections.singletonList(call.argument("trip_id").toString()));
                    break;
                case "stopMultiTracking":
                    stopMultiTracking(call.argument("trip_ids").toString());
                    break;
                case "setTripListener":
                    Teliver.setTripListener(this);
                    break;
                case "sendEventPush":
                    sendEventPush(call.argument("trip_id").toString(), call.argument("push_obj").toString());
                    break;
                case "sendEventPushWithTag":
                    sendEventPush(call.argument("trip_id").toString(), call.argument("push_0bj").toString()
                            , call.argument("tag").toString());
                    break;
                case "tagLocation":
                    tagLocation(call.argument("trip_id").toString(), call.argument("tag").toString());
                    break;
                case "setLogVisibility":
                    setLogVisibility(Boolean.parseBoolean(call.argument("is_log_visible").toString()));
                    break;
                default:
                    result.notImplemented();
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private Gson getGson() {
        return new GsonBuilder().create();
    }

    private void showLogMessage(String msg) {
        Log.v("Telvier Module :: ", ":: " + msg);
    }

    private void identifyUser(String userObj) {
        try {
            if (TUtils.clearNull(userObj).isEmpty())
                showLogMessage("User Object should not be empty or null !");
            else
                Teliver.identifyUser(getGson().fromJson(userObj, User.class));

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void startTracking(String trackingId) {
        try {
            if (TUtils.clearNull(trackingId).isEmpty())
                showLogMessage("Tracking ID should not be empty or null !");
            else {
                MarkerOption option = new MarkerOption(trackingId);
                int resId = context.getResources().getIdentifier("ic_bike", "drawable", context.getPackageName());
                option.setIconMarker(resId);
                List<MarkerOption> options = new ArrayList<>();
                options.add(option);
                Teliver.startTracking(new TrackingBuilder(options)
                        .withTitle("#" + trackingId).build());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void startMultiTracking(String trackingIds) {
        try {
            if (TUtils.clearNull(trackingIds).isEmpty())
                showLogMessage("Tracking IDs should not be empty or null !");
            else {
                List<MarkerOption> options = new ArrayList<>();
                for (String trackingId : trackingIds.split(",")) {
                    MarkerOption option = new MarkerOption(trackingId);

                    int resId = context.getResources().getIdentifier("ic_bike", "drawable", context.getPackageName());
                    option.setIconMarker(resId);
                    options.add(option);
                }
                Teliver.startTracking(new TrackingBuilder(options).build());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void stopTracking(List<String> allIds) {
        Teliver.stopTracking(allIds);
    }

    public void stopMultiTracking(String trackingIds) {
        try {
            if (TUtils.clearNull(trackingIds).isEmpty())
                showLogMessage("Tracking IDs should not be empty or null !");
            else
                stopTracking(Arrays.asList(trackingIds.split(",")));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void sendEventPush(String trackingId, String pushObj) {
        try {
            if (TUtils.clearNull(pushObj).isEmpty())
                showLogMessage("Push Data should not be empty or null !");
            else
                Teliver.sendEventPush(trackingId, getGson().fromJson(
                        pushObj, PushData.class));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void sendEventPush(String trackingId, String pushObj, String tag) {
        try {
            if (TUtils.clearNull(pushObj).isEmpty())
                showLogMessage("Push Data should not be empty or null !");
            else {
                sendEventPush(trackingId, pushObj);
                tagLocation(trackingId, tag);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void tagLocation(String trackingId, String tag) {
        Teliver.tagLocation(trackingId, tag);
    }

    public void setLogVisibility(boolean visibility) {
        TLog.setVisible(visibility);
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
    }

    // connection Listener
    @Override
    public void onSuccess() {
        sendCallBack(true, "Init Success");
    }

    @Override
    public void OnFailure(String reason) {
        sendCallBack(false, "Init Failure::" + reason);
    }

    // trip status listener
    @Override
    public void onTripStarted(Trip tripDetails) {
        sendCallBack(true, "", "onTripStarted"
                , tripDetails.getTrackingId(), getGson().toJson(tripDetails));
    }

    @Override
    public void onLocationUpdate(Location location) {

    }

    @Override
    public void onTripEnded(String trackingID) {
        sendCallBack(true, "", "onTripEnded", trackingID, "");
    }

    @Override
    public void onTripError(String reason) {
        sendCallBack(false, reason, "onTripError", "", "");
    }

    private void sendCallBack(boolean success, String message, String type, String trackingId, String tripObj) {
        try {
            if (result != null) {
                JSONObject jsonObj = new JSONObject();
                jsonObj.put("success", success);
                jsonObj.put("message", message);
                jsonObj.put("type", type);
                jsonObj.put("trackingId", trackingId);
                jsonObj.put("trip", tripObj);
                result.success(jsonObj.toString());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void sendCallBack(boolean success, String message) {
        try {
            if (result != null) {
                JSONObject jsonObj = new JSONObject();
                jsonObj.put("success", success);
                jsonObj.put("message", message);
                result.success(jsonObj.toString());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        this.context = binding.getActivity();
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {

    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {

    }

    @Override
    public void onDetachedFromActivity() {

    }

}
