import 'dart:convert';

import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';

import 'test_plugin_fftt_test_platform_interface.dart';

/// An implementation of [TestPluginFfttTestPlatform] that uses method channels.
class MethodChannelTestPluginFfttTest extends TestPluginFfttTestPlatform {
  /// The method channel used to interact with the native platform.
  @visibleForTesting
  final methodChannel = const MethodChannel('test_plugin_fftt_test');

  @override
  Future<dynamic> initSDK(String key) async {
    dynamic version = await methodChannel
        .invokeMethod('init', <String, dynamic>{"public_key": key});
    return json.decode(version);
  }

  @override
  startTracking(String tripId) {
    return methodChannel
        .invokeMethod("startTracking", <String, dynamic>{"trip_id": tripId});
  }

  @override
  stopTracking(String tripId) {
    return methodChannel
        .invokeMethod("stopTracking", <String, dynamic>{"trip_id": tripId});
  }

  @override
  Future<dynamic> startTrip(String tripId) async {
    dynamic version = await methodChannel
        .invokeMethod('startTrip', <String, dynamic>{"trip_id": tripId});
    return json.decode(version);
  }

  @override
  Future<dynamic> stopTrip(String tripId) async {
    dynamic version = await methodChannel
        .invokeMethod("stopTrip", <String, dynamic>{"trip_id": tripId});
    return json.decode(version);
  }

  @override
  identifyUser(String userData) {
    return methodChannel
        .invokeMethod("identifyUser", <String, dynamic>{"user_data": userData});
  }

  @override
  unregisterUser() {
    return methodChannel.invokeMethod("unregisterUser");
  }

  @override
  startMultiTracking(String tripIds) {
    return methodChannel.invokeMethod(
        "startMultiTracking", <String, dynamic>{"trip_ids": tripIds});
  }

  @override
  stopMultiTracking(String tripIds) {
    return methodChannel.invokeMethod(
        "stopMultiTracking", <String, dynamic>{"trip_ids": tripIds});
  }

  @override
  Future<dynamic> setTripListener() {
    return methodChannel.invokeMethod("setTripListener");
  }

  @override
  sendEventPush(String tripId, String pushData) {
    return methodChannel.invokeMethod(
        "sendEventPUsh", {"trip_id": tripId, "push_obj": pushData});
  }

  @override
  sendEventPushWithTag(String tripId, String pushData, String tagMsg) {
    return methodChannel.invokeMethod("sendEventPushWithTag",
        {"trip_id": tripId, "push_obj": pushData, "tag": tagMsg});
  }

  @override
  tagLocation(String tripId, String tagMsg) {
    return methodChannel
        .invokeMethod("tagLocation", {"trip_id": tripId, "tag": tagMsg});
  }

  @override
  setLogVisible(bool isLogVisible) {
    return methodChannel.invokeMethod(
        "setLogVisibility", <String, dynamic>{"is_log_visible": isLogVisible});
  }
}
